# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.host as host

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    call,
    patch,
)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class HostTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Validate the host state
    '''
    def setup_loader_modules(self):
        return {
            host: {
                '__opts__': {
                    'test': False,
                },
            },
        }

    def test_present(self):
        '''
        Test to ensures that the named host is present with the given ip
        '''
        add_host = MagicMock(return_value=True)
        rm_host = MagicMock(return_value=True)
        hostname = 'salt'
        ip_str = '127.0.0.1'
        ip_list = ['10.1.2.3', '10.4.5.6']

        # Case 1: No match for hostname. Single IP address passed to the state.
        list_hosts = MagicMock(return_value={
            '127.0.0.1': ['localhost'],
        })
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_str)
            assert ret['result'] is True
            assert ret['comment'] == 'Added host {0} ({1})'.format(hostname, ip_str), ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_str: [hostname],
                }
            }, ret['changes']
            expected = [call(ip_str, hostname)]
            assert add_host.mock_calls == expected, add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 2: No match for hostname. Multiple IP addresses passed to the
        # state.
        list_hosts = MagicMock(return_value={
            '127.0.0.1': ['localhost'],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_list[0]) in ret['comment']
            assert 'Added host {0} ({1})'.format(hostname, ip_list[1]) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_list[0]: [hostname],
                    ip_list[1]: [hostname],
                }
            }, ret['changes']
            expected = sorted([call(x, hostname) for x in ip_list])
            assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 3: Match for hostname, but no matching IP. Single IP address
        # passed to the state.
        list_hosts = MagicMock(return_value={
            '127.0.0.1': ['localhost'],
            ip_list[0]: [hostname],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_str)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_str) in ret['comment']
            assert 'Host {0} present for IP address {1}'.format(hostname, ip_list[0]) in ret['warnings'][0]
            assert ret['changes'] == {
                'added': {
                    ip_str: [hostname],
                },
            }, ret['changes']
            expected = [call(ip_str, hostname)]
            assert add_host.mock_calls == expected, add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 3a: Repeat the above with clean=True
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_str, clean=True)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_str) in ret['comment']
            assert 'Removed host {0} ({1})'.format(hostname, ip_list[0]) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_str: [hostname],
                },
                'removed': {
                    ip_list[0]: [hostname],
                }
            }, ret['changes']
            expected = [call(ip_str, hostname)]
            assert add_host.mock_calls == expected, add_host.mock_calls
            expected = [call(ip_list[0], hostname)]
            assert rm_host.mock_calls == expected, rm_host.mock_calls

        # Case 4: Match for hostname, but no matching IP. Multiple IP addresses
        # passed to the state.
        cur_ip = '1.2.3.4'
        list_hosts = MagicMock(return_value={
            '127.0.0.1': ['localhost'],
            cur_ip: [hostname],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_list[0]) in ret['comment']
            assert 'Added host {0} ({1})'.format(hostname, ip_list[1]) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_list[0]: [hostname],
                    ip_list[1]: [hostname],
                },
            }, ret['changes']
            expected = sorted([call(x, hostname) for x in ip_list])
            assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 4a: Repeat the above with clean=True
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list, clean=True)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_list[0]) in ret['comment']
            assert 'Added host {0} ({1})'.format(hostname, ip_list[1]) in ret['comment']
            assert 'Removed host {0} ({1})'.format(hostname, cur_ip) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_list[0]: [hostname],
                    ip_list[1]: [hostname],
                },
                'removed': {
                    cur_ip: [hostname],
                }
            }, ret['changes']
            expected = sorted([call(x, hostname) for x in ip_list])
            assert sorted(add_host.mock_calls) == expected, add_host.mock_calls
            expected = [call(cur_ip, hostname)]
            assert rm_host.mock_calls == expected, rm_host.mock_calls

        # Case 5: Multiple IP addresses passed to the state. One of them
        # matches, the other does not. There is also a non-matching IP that
        # must be removed.
        cur_ip = '1.2.3.4'
        list_hosts = MagicMock(return_value={
            '127.0.0.1': ['localhost'],
            cur_ip: [hostname],
            ip_list[0]: [hostname],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_list[1]) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_list[1]: [hostname],
                },
            }, ret['changes']
            expected = [call(ip_list[1], hostname)]
            assert add_host.mock_calls == expected, add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 5a: Repeat the above with clean=True
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list, clean=True)
            assert ret['result'] is True
            assert 'Added host {0} ({1})'.format(hostname, ip_list[1]) in ret['comment']
            assert 'Removed host {0} ({1})'.format(hostname, cur_ip) in ret['comment']
            assert ret['changes'] == {
                'added': {
                    ip_list[1]: [hostname],
                },
                'removed': {
                    cur_ip: [hostname],
                }
            }, ret['changes']
            expected = [call(ip_list[1], hostname)]
            assert add_host.mock_calls == expected, add_host.mock_calls
            expected = [call(cur_ip, hostname)]
            assert rm_host.mock_calls == expected, rm_host.mock_calls

        # Case 6: Single IP address passed to the state, which matches the
        # current configuration for that hostname. No changes should be made.
        list_hosts = MagicMock(return_value={
            ip_str: [hostname],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_str)
            assert ret['result'] is True
            assert ret['comment'] == 'Host {0} ({1}) already present'.format(hostname, ip_str) in ret['comment']
            assert ret['changes'] == {}, ret['changes']
            assert add_host.mock_calls == [], add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

        # Case 7: Multiple IP addresses passed to the state, which both match
        # the current configuration for that hostname. No changes should be
        # made.
        list_hosts = MagicMock(return_value={
            ip_list[0]: [hostname],
            ip_list[1]: [hostname],
        })
        add_host.reset_mock()
        rm_host.reset_mock()
        with patch.dict(host.__salt__,
                        {'hosts.list_hosts': list_hosts,
                         'hosts.add_host': add_host,
                         'hosts.rm_host': rm_host}):
            ret = host.present(hostname, ip_list)
            assert ret['result'] is True
            assert 'Host {0} ({1}) already present'.format(hostname, ip_list[0]) in ret['comment']
            assert 'Host {0} ({1}) already present'.format(hostname, ip_list[1]) in ret['comment']
            assert ret['changes'] == {}, ret['changes']
            assert add_host.mock_calls == [], add_host.mock_calls
            assert rm_host.mock_calls == [], rm_host.mock_calls

    def test_absent(self):
        '''
        Test to ensure that the named host is absent
        '''
        ret = {'changes': {},
               'comment': 'Host salt (127.0.0.1) already absent',
               'name': 'salt', 'result': True}

        mock = MagicMock(return_value=False)
        with patch.dict(host.__salt__, {'hosts.has_pair': mock}):
            self.assertDictEqual(host.absent("salt", "127.0.0.1"), ret)

    def test_only_already(self):
        '''
        Test only() when the state hasn't changed
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': True,
            'comment': 'IP address 127.0.1.1 already set to "foo.bar"'}
        mock1 = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock1}):
            mock2 = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock2}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", 'foo.bar'))

    def test_only_dryrun(self):
        '''
        Test only() when state would change, but it's a dry run
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': None,
            'comment': 'Would change 127.0.1.1 from "foo.bar" to "foo.bar foo"'}
        mock1 = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock1}):
            mock2 = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock2}):
                with patch.dict(host.__opts__, {'test': True}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))

    def test_only_fail(self):
        '''
        Test only() when state change fails
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {},
            'result': False,
            'comment': 'hosts.set_host failed to change 127.0.1.1'
                + ' from "foo.bar" to "foo.bar foo"'}
        mock = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock}):
            mock = MagicMock(return_value=False)
            with patch.dict(host.__salt__, {'hosts.set_host': mock}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))

    def test_only_success(self):
        '''
        Test only() when state successfully changes
        '''
        expected = {
            'name': '127.0.1.1',
            'changes': {'127.0.1.1': {'old': 'foo.bar', 'new': 'foo.bar foo'}},
            'result': True,
            'comment': 'successfully changed 127.0.1.1'
                + ' from "foo.bar" to "foo.bar foo"'}
        mock = MagicMock(return_value=['foo.bar'])
        with patch.dict(host.__salt__, {'hosts.get_alias': mock}):
            mock = MagicMock(return_value=True)
            with patch.dict(host.__salt__, {'hosts.set_host': mock}):
                with patch.dict(host.__opts__, {'test': False}):
                    self.assertDictEqual(
                        expected,
                        host.only("127.0.1.1", ['foo.bar', 'foo']))



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase

# Import Salt libs
import salt.renderers.yaml as yaml


class YAMLRendererTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        return {yaml: {}}

    def test_yaml_render_string(self):
        data = 'string'
        result = yaml.render(data)

        self.assertEqual(result, data)

    def test_yaml_render_unicode(self):
        data = '!!python/unicode python unicode string'
        result = yaml.render(data)

        self.assertEqual(result, u'python unicode string')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    patch,
    NO_MOCK,
    NO_MOCK_REASON
)

# Import Salt Libs
import salt.states.win_network as win_network


@skipIf(NO_MOCK, NO_MOCK_REASON)
class WinNetworkTestCase(TestCase, LoaderModuleMockMixin):
    '''
        Validate the nftables state
    '''
    def setup_loader_modules(self):
        return {win_network: {}}

    def test_managed_missing_parameters(self):
        '''
        Test to ensure that the named interface is configured properly.
        '''
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'dns_proto must be one of the following: static, dhcp\n'
                          'ip_proto must be one of the following: static, dhcp'}
        self.assertDictEqual(win_network.managed('salt'), ret)

    def test_managed_static_enabled_false(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': 'Interface \'salt\' is up to date (already disabled)'}
        mock_false = MagicMock(return_value=False)
        with patch.dict(win_network.__salt__, {"ip.is_enabled": mock_false}):
            self.assertDictEqual(
                win_network.managed(
                    'salt', dns_proto='static', ip_proto='static', enabled=False),
                ret)

    def test_managed_test_true(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'Failed to enable interface \'salt\' to make changes'}
        mock_false = MagicMock(return_value=False)
        with patch.dict(win_network.__salt__, {"ip.is_enabled": mock_false,
                                               "ip.enable": mock_false}), \
                patch.dict(win_network.__opts__, {"test": False}):
            self.assertDictEqual(
                win_network.managed(
                    'salt', dns_proto='static', ip_proto='static'),
                ret)

    def test_managed_validate_errors(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'The following SLS configuration errors were '
                          'detected:\n'
                          '- First Error\n'
                          '- Second Error'}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=['First Error', 'Second Error'])
        with patch.dict(win_network.__salt__, {"ip.is_enabled": mock_true}),\
                patch.object(win_network, '_validate', mock_validate):
            self.assertDictEqual(
                win_network.managed(
                    'salt', dns_proto='static', ip_proto='static'),
                ret)

    def test_managed_get_current_config_failed(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'Unable to get current configuration for interface '
                          '\'salt\''}
        mock_true = MagicMock(return_value=True)
        mock_false = MagicMock(return_value=False)
        mock_validate = MagicMock(return_value=[])
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_false}), \
                patch.object(win_network, '_validate', mock_validate):

            self.assertDictEqual(
                win_network.managed('salt', dns_proto='dhcp', ip_proto='dhcp'),
                ret)

    def test_managed_test_true_no_changes(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': True,
               'comment': 'Interface \'salt\' is up to date'}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(return_value={
            'DHCP enabled': 'yes',
            'DNS servers configured through DHCP': '192.168.0.10'})
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int}), \
                patch.dict(win_network.__opts__, {"test": True}), \
                patch.object(win_network, '_validate', mock_validate):
            self.assertDictEqual(
                win_network.managed('salt', dns_proto='dhcp', ip_proto='dhcp'),
                ret)

    def test_managed_test_true_changes(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': None,
               'comment': 'The following changes will be made to interface '
                          '\'salt\':\n'
                          '- DNS protocol will be changed to: dhcp'}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(return_value={
            'DHCP enabled': 'no',
            'Statically Configured DNS Servers': '192.168.0.10'})
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int}), \
                patch.dict(win_network.__opts__, {"test": True}), \
                patch.object(win_network, '_validate', mock_validate):

            self.assertDictEqual(
                win_network.managed('salt', dns_proto='dhcp', ip_proto='dhcp'),
                ret)

    def test_managed_failed(self):
        ret = {'name': 'salt',
               'changes': {},
               'result': False,
               'comment': 'Failed to set desired configuration settings for '
                          'interface \'salt\''}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(return_value={
            'DHCP enabled': 'no',
            'Statically Configured DNS Servers': '192.168.0.10'})
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int,
                                               'ip.set_dhcp_dns': mock_true,
                                               'ip.set_dhcp_ip': mock_true}), \
                patch.dict(win_network.__opts__, {"test": False}), \
                patch.object(win_network, '_validate', mock_validate):
            self.assertDictEqual(
                win_network.managed('salt', dns_proto='dhcp', ip_proto='dhcp'),
                ret)

    def test_managed(self):
        ret = {'name': 'salt',
               'changes': {
                   'DHCP enabled': {
                       'new': 'yes',
                       'old': 'no'},
                   'DNS servers configured through DHCP': {
                       'new': '192.168.0.10',
                       'old': ''},
                   'Statically Configured DNS Servers': {
                       'new': '',
                       'old': '192.168.0.10'
                   }
               },
               'result': True,
               'comment': 'Successfully updated configuration for interface '
                          '\'salt\''}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(side_effect=[
            {'DHCP enabled': 'no', 'Statically Configured DNS Servers': '192.168.0.10'},
            {'DHCP enabled': 'yes', 'DNS servers configured through DHCP': '192.168.0.10'},
        ])
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int,
                                               'ip.set_dhcp_dns': mock_true,
                                               'ip.set_dhcp_ip': mock_true}), \
             patch.dict(win_network.__opts__, {"test": False}), \
             patch.object(win_network, '_validate', mock_validate):
            self.assertDictEqual(
                win_network.managed('salt', dns_proto='dhcp', ip_proto='dhcp'),
                ret)

    def test_managed_static_dns_clear(self):
        expected = {'name': 'salt',
                    'changes': {
                        'Statically Configured DNS Servers': {
                            'new': 'None',
                            'old': '192.168.0.10'
                        }
                    },
                    'result': True,
                    'comment': 'Successfully updated configuration for '
                               'interface \'salt\''}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(side_effect=[
            {'DHCP enabled': 'no', 'Statically Configured DNS Servers': '192.168.0.10'},
            {'DHCP enabled': 'no', 'Statically Configured DNS Servers': 'None'},
        ])
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int,
                                               'ip.set_static_dns': mock_true}), \
                patch.dict(win_network.__opts__, {"test": False}), \
                patch.object(win_network, '_validate', mock_validate):
            ret = win_network.managed(
                'salt', dns_proto='static', dns_servers=[], ip_proto='dhcp')
            self.assertDictEqual(ret, expected)

    def test_managed_static_dns(self):
        expected = {'name': 'salt',
                    'changes': {
                        'Statically Configured DNS Servers': {
                            'new': '192.168.0.10',
                            'old': 'None'
                        }
                    },
                    'result': True,
                    'comment': 'Successfully updated configuration for '
                               'interface \'salt\''}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(side_effect=[
            {'DHCP enabled': 'no', 'Statically Configured DNS Servers': 'None'},
            {'DHCP enabled': 'no', 'Statically Configured DNS Servers': '192.168.0.10'},
        ])
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int,
                                               'ip.set_static_dns': mock_true}), \
                patch.dict(win_network.__opts__, {"test": False}), \
                patch.object(win_network, '_validate', mock_validate):
            ret = win_network.managed(
                'salt', dns_proto='static', dns_servers=['192.168.0.10'], ip_proto='dhcp')
            self.assertDictEqual(ret, expected)

    def test_managed_static_dns_no_action(self):
        expected = {'name': 'salt',
                    'changes': {},
                    'result': True,
                    'comment': 'Interface \'salt\' is up to date'}
        mock_true = MagicMock(return_value=True)
        mock_validate = MagicMock(return_value=[])
        mock_get_int = MagicMock(return_value={
            'DHCP enabled': 'no',
            'Statically Configured DNS Servers': '192.168.0.10'
        })
        with patch.dict(win_network.__salt__, {'ip.is_enabled': mock_true,
                                               'ip.get_interface': mock_get_int,
                                               'ip.set_static_dns': mock_true}), \
                patch.dict(win_network.__opts__, {"test": False}), \
                patch.object(win_network, '_validate', mock_validate):
            # Don't pass dns_servers
            ret = win_network.managed('salt', dns_proto='static', ip_proto='dhcp')
            self.assertDictEqual(ret, expected)
            # Pass dns_servers=None
            ret = win_network.managed(
                'salt', dns_proto='static', dns_servers=None, ip_proto='dhcp')
            self.assertDictEqual(ret, expected)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Mike Place <mp@saltstack.com>
'''

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import copy
import os
import tempfile

# Import Salt Testing libs
from tests.integration import AdaptedConfigurationTestCaseMixin
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.paths import BASE_FILES, TMP, TMP_STATE_TREE
from tests.support.unit import TestCase, skipIf
from tests.support.mock import patch, NO_MOCK, NO_MOCK_REASON

# Import Salt libs
import salt.fileserver.roots as roots
import salt.fileclient
import salt.utils.files
import salt.utils.hashutils
import salt.utils.platform

try:
    import win32file
except ImportError:
    pass

UNICODE_FILENAME = 'питон.txt'
UNICODE_DIRNAME = UNICODE_ENVNAME = 'соль'


@skipIf(NO_MOCK, NO_MOCK_REASON)
class RootsTest(TestCase, AdaptedConfigurationTestCaseMixin, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        self.tmp_cachedir = tempfile.mkdtemp(dir=TMP)
        self.opts = self.get_temp_config('master')
        self.opts['cachedir'] = self.tmp_cachedir
        empty_dir = os.path.join(TMP_STATE_TREE, 'empty_dir')
        if not os.path.isdir(empty_dir):
            os.makedirs(empty_dir)
        return {roots: {'__opts__': self.opts}}

    @classmethod
    def setUpClass(cls):
        '''
        Create special file_roots for symlink test on Windows
        '''
        if salt.utils.platform.is_windows():
            root_dir = tempfile.mkdtemp(dir=TMP)
            source_sym = os.path.join(root_dir, 'source_sym')
            with salt.utils.files.fopen(source_sym, 'w') as fp_:
                fp_.write('hello world!\n')
            cwd = os.getcwd()
            try:
                os.chdir(root_dir)
                win32file.CreateSymbolicLink('dest_sym', 'source_sym', 0)
            finally:
                os.chdir(cwd)
            cls.test_symlink_list_file_roots = {'base': [root_dir]}
        else:
            cls.test_symlink_list_file_roots = None
        cls.tmp_dir = tempfile.mkdtemp(dir=TMP)
        full_path_to_file = os.path.join(BASE_FILES, 'testfile')
        with salt.utils.files.fopen(full_path_to_file, 'rb') as s_fp:
            with salt.utils.files.fopen(os.path.join(cls.tmp_dir, 'testfile'), 'wb') as d_fp:
                for line in s_fp:
                    d_fp.write(line)

    @classmethod
    def tearDownClass(cls):
        '''
        Remove special file_roots for symlink test
        '''
        if salt.utils.platform.is_windows():
            try:
                salt.utils.files.rm_rf(cls.test_symlink_list_file_roots['base'][0])
            except OSError:
                pass
        salt.utils.files.rm_rf(cls.tmp_dir)

    def tearDown(self):
        del self.opts

    def test_file_list(self):
        ret = roots.file_list({'saltenv': 'base'})
        self.assertIn('testfile', ret)
        self.assertIn(UNICODE_FILENAME, ret)

    def test_find_file(self):
        ret = roots.find_file('testfile')
        self.assertEqual('testfile', ret['rel'])

        full_path_to_file = os.path.join(BASE_FILES, 'testfile')
        self.assertEqual(full_path_to_file, ret['path'])

    def test_serve_file(self):
        with patch.dict(roots.__opts__, {'file_buffer_size': 262144}):
            load = {'saltenv': 'base',
                    'path': os.path.join(self.tmp_dir, 'testfile'),
                    'loc': 0
                    }
            fnd = {'path': os.path.join(self.tmp_dir, 'testfile'),
                   'rel': 'testfile'}
            ret = roots.serve_file(load, fnd)

            with salt.utils.files.fopen(
                    os.path.join(BASE_FILES, 'testfile'), 'rb') as fp_:
                data = fp_.read()

            self.assertDictEqual(
                ret,
                {'data': data,
                 'dest': 'testfile'})

    def test_envs(self):
        opts = {'file_roots': copy.copy(self.opts['file_roots'])}
        opts['file_roots'][UNICODE_ENVNAME] = opts['file_roots']['base']
        with patch.dict(roots.__opts__, opts):
            ret = roots.envs()
        self.assertIn('base', ret)
        self.assertIn(UNICODE_ENVNAME, ret)

    def test_file_hash(self):
        load = {
                'saltenv': 'base',
                'path': os.path.join(self.tmp_dir, 'testfile'),
                }
        fnd = {
            'path': os.path.join(self.tmp_dir, 'testfile'),
            'rel': 'testfile'
        }
        ret = roots.file_hash(load, fnd)

        # Hashes are different in Windows. May be how git translates line
        # endings
        with salt.utils.files.fopen(
                os.path.join(BASE_FILES, 'testfile'), 'rb') as fp_:
            hsum = salt.utils.hashutils.sha256_digest(fp_.read())

        self.assertDictEqual(
            ret,
            {
                'hsum': hsum,
                'hash_type': 'sha256'
            }
        )

    def test_file_list_emptydirs(self):
        ret = roots.file_list_emptydirs({'saltenv': 'base'})
        self.assertIn('empty_dir', ret)

    def test_file_list_with_slash(self):
        opts = {'file_roots': copy.copy(self.opts['file_roots'])}
        opts['file_roots']['foo/bar'] = opts['file_roots']['base']
        load = {
                'saltenv': 'foo/bar',
                }
        with patch.dict(roots.__opts__, opts):
            ret = roots.file_list(load)
        self.assertIn('testfile', ret)
        self.assertIn(UNICODE_FILENAME, ret)

    def test_dir_list(self):
        ret = roots.dir_list({'saltenv': 'base'})
        self.assertIn('empty_dir', ret)
        self.assertIn(UNICODE_DIRNAME, ret)

    def test_symlink_list(self):
        orig_file_roots = self.opts['file_roots']
        try:
            if self.test_symlink_list_file_roots:
                self.opts['file_roots'] = self.test_symlink_list_file_roots
            ret = roots.symlink_list({'saltenv': 'base'})
            self.assertDictEqual(ret, {'dest_sym': 'source_sym'})
        finally:
            if self.test_symlink_list_file_roots:
                self.opts['file_roots'] = orig_file_roots



# -*- coding: utf-8 -*-
'''
    :synopsis: Unit Tests for 'module.aptkernelpkg'
    :platform: Linux
    :maturity: develop
    versionadded:: 2018.3.0
'''
# pylint: disable=invalid-name,no-member

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt libs
from salt.ext import six

try:
    # Import Salt Testing Libs
    from tests.support.mixins import LoaderModuleMockMixin
    from tests.support.unit import skipIf, TestCase
    from tests.support.mock import (
        NO_MOCK,
        NO_MOCK_REASON,
        MagicMock,
        patch)

    # Import Salt Libs
    import salt.states.kernelpkg as kernelpkg
    HAS_MODULES = True
except ImportError:
    HAS_MODULES = False

KERNEL_LIST = ['4.4.0-70-generic', '4.4.0-71-generic', '4.5.1-14-generic']
STATE_NAME = 'kernelpkg-test'


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_MODULES, 'Salt modules could not be loaded')
class KernelPkgTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.aptpkg
    '''

    def setup_loader_modules(self):
        return {
            kernelpkg: {
                '__salt__': {
                    'system.reboot': MagicMock(return_value=None),
                    'kernelpkg.upgrade': MagicMock(return_value={
                        'upgrades': {
                            'kernel': {
                                'old': '1.0.0',
                                'new': '2.0.0',
                            }
                        }
                    }),
                    'kernelpkg.active': MagicMock(return_value=0),
                    'kernelpkg.latest_installed': MagicMock(return_value=0)
                }
            }
        }

    def test_latest_installed_with_changes(self):
        '''
        Test - latest_installed when an upgrade is available
        '''
        installed = MagicMock(return_value=KERNEL_LIST[:-1])
        upgrade = MagicMock(return_value=KERNEL_LIST[-1])
        with patch.dict(kernelpkg.__salt__, {'kernelpkg.list_installed': installed}):
            with patch.dict(kernelpkg.__salt__, {'kernelpkg.latest_available': upgrade}):
                with patch.dict(kernelpkg.__opts__, {'test': False}):
                    kernelpkg.__salt__['kernelpkg.upgrade'].reset_mock()
                    ret = kernelpkg.latest_installed(name=STATE_NAME)
                    self.assertEqual(ret['name'], STATE_NAME)
                    self.assertTrue(ret['result'])
                    self.assertIsInstance(ret['changes'], dict)
                    self.assertIsInstance(ret['comment'], six.text_type)
                    self.assert_called_once(kernelpkg.__salt__['kernelpkg.upgrade'])

                with patch.dict(kernelpkg.__opts__, {'test': True}):
                    kernelpkg.__salt__['kernelpkg.upgrade'].reset_mock()
                    ret = kernelpkg.latest_installed(name=STATE_NAME)
                    self.assertEqual(ret['name'], STATE_NAME)
                    self.assertIsNone(ret['result'])
                    self.assertDictEqual(ret['changes'], {})
                    self.assertIsInstance(ret['comment'], six.text_type)
                    kernelpkg.__salt__['kernelpkg.upgrade'].assert_not_called()

    def test_latest_installed_at_latest(self):
        '''
        Test - latest_installed when no upgrade is available
        '''
        installed = MagicMock(return_value=KERNEL_LIST)
        upgrade = MagicMock(return_value=KERNEL_LIST[-1])
        with patch.dict(kernelpkg.__salt__, {'kernelpkg.list_installed': installed}):
            with patch.dict(kernelpkg.__salt__, {'kernelpkg.latest_available': upgrade}):
                with patch.dict(kernelpkg.__opts__, {'test': False}):
                    ret = kernelpkg.latest_installed(name=STATE_NAME)
                    self.assertEqual(ret['name'], STATE_NAME)
                    self.assertTrue(ret['result'])
                    self.assertDictEqual(ret['changes'], {})
                    self.assertIsInstance(ret['comment'], six.text_type)
                    kernelpkg.__salt__['kernelpkg.upgrade'].assert_not_called()

                with patch.dict(kernelpkg.__opts__, {'test': True}):
                    ret = kernelpkg.latest_installed(name=STATE_NAME)
                    self.assertEqual(ret['name'], STATE_NAME)
                    self.assertTrue(ret['result'])
                    self.assertDictEqual(ret['changes'], {})
                    self.assertIsInstance(ret['comment'], six.text_type)
                    kernelpkg.__salt__['kernelpkg.upgrade'].assert_not_called()

    def test_latest_active_with_changes(self):
        '''
        Test - latest_active when a new kernel is available
        '''
        reboot = MagicMock(return_value=True)
        with patch.dict(kernelpkg.__salt__, {'kernelpkg.needs_reboot': reboot}):
            with patch.dict(kernelpkg.__opts__, {'test': False}):
                kernelpkg.__salt__['system.reboot'].reset_mock()
                ret = kernelpkg.latest_active(name=STATE_NAME)
                self.assertEqual(ret['name'], STATE_NAME)
                self.assertTrue(ret['result'])
                self.assertIsInstance(ret['changes'], dict)
                self.assertIsInstance(ret['comment'], six.text_type)
                self.assert_called_once(kernelpkg.__salt__['system.reboot'])

            with patch.dict(kernelpkg.__opts__, {'test': True}):
                kernelpkg.__salt__['system.reboot'].reset_mock()
                ret = kernelpkg.latest_active(name=STATE_NAME)
                self.assertEqual(ret['name'], STATE_NAME)
                self.assertIsNone(ret['result'])
                self.assertDictEqual(ret['changes'], {})
                self.assertIsInstance(ret['comment'], six.text_type)
                kernelpkg.__salt__['system.reboot'].assert_not_called()

    def test_latest_active_at_latest(self):
        '''
        Test - latest_active when the newest kernel is already active
        '''
        reboot = MagicMock(return_value=False)
        with patch.dict(kernelpkg.__salt__, {'kernelpkg.needs_reboot': reboot}):
            with patch.dict(kernelpkg.__opts__, {'test': False}):
                kernelpkg.__salt__['system.reboot'].reset_mock()
                ret = kernelpkg.latest_active(name=STATE_NAME)
                self.assertEqual(ret['name'], STATE_NAME)
                self.assertTrue(ret['result'])
                self.assertDictEqual(ret['changes'], {})
                self.assertIsInstance(ret['comment'], six.text_type)
                kernelpkg.__salt__['system.reboot'].assert_not_called()

            with patch.dict(kernelpkg.__opts__, {'test': True}):
                kernelpkg.__salt__['system.reboot'].reset_mock()
                ret = kernelpkg.latest_active(name=STATE_NAME)
                self.assertEqual(ret['name'], STATE_NAME)
                self.assertTrue(ret['result'])
                self.assertDictEqual(ret['changes'], {})
                self.assertIsInstance(ret['comment'], six.text_type)
                kernelpkg.__salt__['system.reboot'].assert_not_called()

    def test_latest_wait(self):
        '''
        Test - latest_wait static results
        '''
        ret = kernelpkg.latest_wait(name=STATE_NAME)
        self.assertEqual(ret['name'], STATE_NAME)
        self.assertTrue(ret['result'])
        self.assertDictEqual(ret['changes'], {})
        self.assertIsInstance(ret['comment'], six.text_type)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
'''

# Import Python Libs
from __future__ import absolute_import, unicode_literals, print_function

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase, skipIf
from tests.support.mock import (
    MagicMock,
    NO_MOCK,
    NO_MOCK_REASON
)

import tests.support.napalm as napalm_test_support
import salt.modules.napalm_snmp as napalm_snmp  # NOQA
import salt.modules.napalm_network as napalm_network  # NOQA


@skipIf(NO_MOCK, NO_MOCK_REASON)
class NapalmSnmpModuleTestCase(TestCase, LoaderModuleMockMixin):

    def setup_loader_modules(self):
        module_globals = {
            '__salt__': {
                'config.option': MagicMock(return_value={
                    'test': {
                        'driver': 'test',
                        'key': '2orgk34kgk34g'
                    }
                }),
                'file.file_exists': napalm_test_support.true,
                'file.join': napalm_test_support.join,
                'file.get_managed': napalm_test_support.get_managed_file,
                'random.hash': napalm_test_support.random_hash,
                'net.load_template': napalm_network.load_template
            }
        }

        return {napalm_snmp: module_globals, napalm_network: module_globals}

    def test_config(self):
        ret = napalm_snmp.config()
        assert ret['out'] == napalm_test_support.TEST_SNMP_INFO

    def test_remove_config(self):
        ret = napalm_snmp.remove_config('1.2.3.4')
        assert ret['result'] is False

    def test_update_config(self):
        ret = napalm_snmp.update_config('1.2.3.4')
        assert ret['result'] is False



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)

    =============
    Class Mix-Ins
    =============

    Some reusable class Mixins
'''
# pylint: disable=repr-flag-used-in-string

# Import python libs
from __future__ import absolute_import, print_function
import os
import sys
import time
import types
import atexit
import pprint
import logging
import tempfile
import functools
import subprocess
import multiprocessing

# Import Salt Testing Libs
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch
from tests.support.runtests import RUNTIME_VARS
from tests.support.paths import CODE_DIR

# Import salt libs
import salt.config
import salt.utils.event
import salt.utils.files
import salt.utils.functools
import salt.utils.path
import salt.utils.stringutils
import salt.utils.yaml
import salt.version
import salt.exceptions
import salt.utils.process
from salt.utils.verify import verify_env
from salt.utils.immutabletypes import freeze
from salt._compat import ElementTree as etree

# Import 3rd-party libs
from salt.ext import six
from salt.ext.six.moves import zip  # pylint: disable=import-error,redefined-builtin

log = logging.getLogger(__name__)


class CheckShellBinaryNameAndVersionMixin(object):
    '''
    Simple class mix-in to subclass in companion to :class:`ShellTestCase<tests.support.case.ShellTestCase>` which
    adds a test case to verify proper version report from Salt's CLI tools.
    '''

    _call_binary_ = None
    _call_binary_expected_version_ = None

    def test_version_includes_binary_name(self):
        if getattr(self, '_call_binary_', None) is None:
            self.skipTest('\'_call_binary_\' not defined.')

        if self._call_binary_expected_version_ is None:
            # Late import
            self._call_binary_expected_version_ = salt.version.__version__

        out = '\n'.join(self.run_script(self._call_binary_, '--version'))
        self.assertIn(self._call_binary_, out)
        self.assertIn(self._call_binary_expected_version_, out)


class AdaptedConfigurationTestCaseMixin(object):

    __slots__ = ()

    @staticmethod
    def get_temp_config(config_for, **config_overrides):
        rootdir = tempfile.mkdtemp(dir=RUNTIME_VARS.TMP)
        conf_dir = os.path.join(rootdir, 'conf')
        for key in ('cachedir', 'pki_dir', 'sock_dir'):
            if key not in config_overrides:
                config_overrides[key] = key
        if 'log_file' not in config_overrides:
            config_overrides['log_file'] = 'logs/{}.log'.format(config_for)
        if 'user' not in config_overrides:
            config_overrides['user'] = RUNTIME_VARS.RUNNING_TESTS_USER
        config_overrides['root_dir'] = rootdir

        cdict = AdaptedConfigurationTestCaseMixin.get_config(config_for, from_scratch=True)

        if config_for in ('master', 'client_config'):
            rdict = salt.config.apply_master_config(config_overrides, cdict)
        if config_for == 'minion':
            rdict = salt.config.apply_minion_config(config_overrides, cdict)

        verify_env([os.path.join(rdict['pki_dir'], 'minions'),
                    os.path.join(rdict['pki_dir'], 'minions_pre'),
                    os.path.join(rdict['pki_dir'], 'minions_rejected'),
                    os.path.join(rdict['pki_dir'], 'minions_denied'),
                    os.path.join(rdict['cachedir'], 'jobs'),
                    os.path.join(rdict['cachedir'], 'raet'),
                    os.path.join(rdict['cachedir'], 'tokens'),
                    os.path.join(rdict['root_dir'], 'cache', 'tokens'),
                    os.path.join(rdict['pki_dir'], 'accepted'),
                    os.path.join(rdict['pki_dir'], 'rejected'),
                    os.path.join(rdict['pki_dir'], 'pending'),
                    os.path.dirname(rdict['log_file']),
                    rdict['sock_dir'],
                    conf_dir
                   ],
                   RUNTIME_VARS.RUNNING_TESTS_USER,
                   root_dir=rdict['root_dir'],
                   )

        rdict['config_dir'] = conf_dir
        rdict['conf_file'] = os.path.join(conf_dir, config_for)
        with salt.utils.files.fopen(rdict['conf_file'], 'w') as wfh:
            salt.utils.yaml.safe_dump(rdict, wfh, default_flow_style=False)
        return rdict

    @staticmethod
    def get_config(config_for, from_scratch=False):
        if from_scratch:
            if config_for in ('master', 'syndic_master'):
                return salt.config.master_config(
                    AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for)
                )
            elif config_for in ('minion', 'sub_minion'):
                return salt.config.minion_config(
                    AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for)
                )
            elif config_for in ('syndic',):
                return salt.config.syndic_config(
                    AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for),
                    AdaptedConfigurationTestCaseMixin.get_config_file_path('minion')
                )
            elif config_for == 'client_config':
                return salt.config.client_config(
                    AdaptedConfigurationTestCaseMixin.get_config_file_path('master')
                )

        if config_for not in RUNTIME_VARS.RUNTIME_CONFIGS:
            if config_for in ('master', 'syndic_master'):
                RUNTIME_VARS.RUNTIME_CONFIGS[config_for] = freeze(
                    salt.config.master_config(
                        AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for)
                    )
                )
            elif config_for in ('minion', 'sub_minion'):
                RUNTIME_VARS.RUNTIME_CONFIGS[config_for] = freeze(
                    salt.config.minion_config(
                        AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for)
                    )
                )
            elif config_for in ('syndic',):
                RUNTIME_VARS.RUNTIME_CONFIGS[config_for] = freeze(
                    salt.config.syndic_config(
                        AdaptedConfigurationTestCaseMixin.get_config_file_path(config_for),
                        AdaptedConfigurationTestCaseMixin.get_config_file_path('minion')
                    )
                )
            elif config_for == 'client_config':
                RUNTIME_VARS.RUNTIME_CONFIGS[config_for] = freeze(
                    salt.config.client_config(
                        AdaptedConfigurationTestCaseMixin.get_config_file_path('master')
                    )
                )
        return RUNTIME_VARS.RUNTIME_CONFIGS[config_for]

    @staticmethod
    def get_config_dir():
        return RUNTIME_VARS.TMP_CONF_DIR

    @staticmethod
    def get_config_file_path(filename):
        if filename == 'syndic_master':
            return os.path.join(RUNTIME_VARS.TMP_SYNDIC_MASTER_CONF_DIR, 'master')
        if filename == 'syndic':
            return os.path.join(RUNTIME_VARS.TMP_SYNDIC_MINION_CONF_DIR, 'minion')
        if filename == 'sub_minion':
            return os.path.join(RUNTIME_VARS.TMP_SUB_MINION_CONF_DIR, 'minion')
        return os.path.join(RUNTIME_VARS.TMP_CONF_DIR, filename)

    @property
    def master_opts(self):
        '''
        Return the options used for the master
        '''
        return self.get_config('master')

    @property
    def minion_opts(self):
        '''
        Return the options used for the minion
        '''
        return self.get_config('minion')

    @property
    def sub_minion_opts(self):
        '''
        Return the options used for the sub_minion
        '''
        return self.get_config('sub_minion')


class SaltClientTestCaseMixin(AdaptedConfigurationTestCaseMixin):
    '''
    Mix-in class that provides a ``client`` attribute which returns a Salt
    :class:`LocalClient<salt:salt.client.LocalClient>`.

    .. code-block:: python

        class LocalClientTestCase(TestCase, SaltClientTestCaseMixin):

            def test_check_pub_data(self):
                just_minions = {'minions': ['m1', 'm2']}
                jid_no_minions = {'jid': '1234', 'minions': []}
                valid_pub_data = {'minions': ['m1', 'm2'], 'jid': '1234'}

                self.assertRaises(EauthAuthenticationError,
                                  self.client._check_pub_data, None)
                self.assertDictEqual({},
                    self.client._check_pub_data(just_minions),
                    'Did not handle lack of jid correctly')

                self.assertDictEqual(
                    {},
                    self.client._check_pub_data({'jid': '0'}),
                    'Passing JID of zero is not handled gracefully')
    '''
    _salt_client_config_file_name_ = 'master'

    @property
    def client(self):
        # Late import
        import salt.client
        if 'runtime_client' not in RUNTIME_VARS.RUNTIME_CONFIGS:
            mopts = self.get_config(self._salt_client_config_file_name_, from_scratch=True)
            RUNTIME_VARS.RUNTIME_CONFIGS['runtime_client'] = salt.client.get_local_client(mopts=mopts)
        return RUNTIME_VARS.RUNTIME_CONFIGS['runtime_client']


class ShellCaseCommonTestsMixin(CheckShellBinaryNameAndVersionMixin):

    _call_binary_expected_version_ = salt.version.__version__

    def test_salt_with_git_version(self):
        if getattr(self, '_call_binary_', None) is None:
            self.skipTest('\'_call_binary_\' not defined.')
        from salt.version import __version_info__, SaltStackVersion
        git = salt.utils.path.which('git')
        if not git:
            self.skipTest('The git binary is not available')
        opts = {
            'stdout': subprocess.PIPE,
            'stderr': subprocess.PIPE,
            'cwd': CODE_DIR,
        }
        if not salt.utils.platform.is_windows():
            opts['close_fds'] = True
        # Let's get the output of git describe
        process = subprocess.Popen(
            [git, 'describe', '--tags', '--first-parent', '--match', 'v[0-9]*'],
            **opts
        )
        out, err = process.communicate()
        if process.returncode != 0:
            process = subprocess.Popen(
                [git, 'describe', '--tags', '--match', 'v[0-9]*'],
                **opts
            )
            out, err = process.communicate()
        if not out:
            self.skipTest(
                'Failed to get the output of \'git describe\'. '
                'Error: \'{0}\''.format(
                    salt.utils.stringutils.to_str(err)
                )
            )

        parsed_version = SaltStackVersion.parse(out)

        if parsed_version.info < __version_info__:
            self.skipTest(
                'We\'re likely about to release a new version. This test '
                'would fail. Parsed(\'{0}\') < Expected(\'{1}\')'.format(
                    parsed_version.info, __version_info__
                )
            )
        elif parsed_version.info != __version_info__:
            self.skipTest(
                'In order to get the proper salt version with the '
                'git hash you need to update salt\'s local git '
                'tags. Something like: \'git fetch --tags\' or '
                '\'git fetch --tags upstream\' if you followed '
                'salt\'s contribute documentation. The version '
                'string WILL NOT include the git hash.'
            )
        out = '\n'.join(self.run_script(self._call_binary_, '--version'))
        self.assertIn(parsed_version.string, out)


class _FixLoaderModuleMockMixinMroOrder(type):
    '''
    This metaclass will make sure that LoaderModuleMockMixin will always come as the first
    base class in order for LoaderModuleMockMixin.setUp to actually run
    '''
    def __new__(mcs, cls_name, cls_bases, cls_dict):
        if cls_name == 'LoaderModuleMockMixin':
            return super(_FixLoaderModuleMockMixinMroOrder, mcs).__new__(mcs, cls_name, cls_bases, cls_dict)
        bases = list(cls_bases)
        for idx, base in enumerate(bases):
            if base.__name__ == 'LoaderModuleMockMixin':
                bases.insert(0, bases.pop(idx))
                break

        # Create the class instance
        instance = super(_FixLoaderModuleMockMixinMroOrder, mcs).__new__(mcs, cls_name, tuple(bases), cls_dict)

        # Apply our setUp function decorator
        instance.setUp = LoaderModuleMockMixin.__setup_loader_modules_mocks__(instance.setUp)
        return instance


class LoaderModuleMockMixin(six.with_metaclass(_FixLoaderModuleMockMixinMroOrder, object)):
    '''
    This class will setup salt loader dunders.

    Please check `set_up_loader_mocks` above
    '''

    # Define our setUp function decorator
    @staticmethod
    def __setup_loader_modules_mocks__(setup_func):

        @functools.wraps(setup_func)
        def wrapper(self):
            if NO_MOCK:
                self.skipTest(NO_MOCK_REASON)

            loader_modules_configs = self.setup_loader_modules()
            if not isinstance(loader_modules_configs, dict):
                raise RuntimeError(
                    '{}.setup_loader_modules() must return a dictionary where the keys are the '
                    'modules that require loader mocking setup and the values, the global module '
                    'variables for each of the module being mocked. For example \'__salt__\', '
                    '\'__opts__\', etc.'.format(self.__class__.__name__)
                )

            salt_dunders = (
                '__opts__', '__salt__', '__runner__', '__context__', '__utils__',
                '__ext_pillar__', '__thorium__', '__states__', '__serializers__', '__ret__',
                '__grains__', '__pillar__', '__sdb__',
                # Proxy is commented out on purpose since some code in salt expects a NameError
                # and is most of the time not a required dunder
                # '__proxy__'
            )

            for module, module_globals in six.iteritems(loader_modules_configs):
                if not isinstance(module, types.ModuleType):
                    raise RuntimeError(
                        'The dictionary keys returned by {}.setup_loader_modules() '
                        'must be an imported module, not {}'.format(
                            self.__class__.__name__,
                            type(module)
                        )
                    )
                if not isinstance(module_globals, dict):
                    raise RuntimeError(
                        'The dictionary values returned by {}.setup_loader_modules() '
                        'must be a dictionary, not {}'.format(
                            self.__class__.__name__,
                            type(module_globals)
                        )
                    )

                module_blacklisted_dunders = module_globals.pop('blacklisted_dunders', ())

                minion_funcs = {}
                if '__salt__' in module_globals and module_globals['__salt__'] == 'autoload':
                    if '__opts__' not in module_globals:
                        raise RuntimeError(
                            'You must provide \'__opts__\' on the {} module globals dictionary '
                            'to auto load the minion functions'.format(module.__name__)
                        )
                    import salt.loader
                    ctx = {}
                    if '__utils__' not in module_globals:
                        utils = salt.loader.utils(module_globals['__opts__'],
                                                  context=module_globals.get('__context__') or ctx)
                        module_globals['__utils__'] = utils
                    minion_funcs = salt.loader.minion_mods(
                        module_globals['__opts__'],
                        context=module_globals.get('__context__') or ctx,
                        utils=module_globals.get('__utils__'),
                    )
                    module_globals['__salt__'] = minion_funcs

                for dunder_name in salt_dunders:
                    if dunder_name not in module_globals:
                        if dunder_name in module_blacklisted_dunders:
                            continue
                        module_globals[dunder_name] = {}

                sys_modules = module_globals.pop('sys.modules', None)
                if sys_modules is not None:
                    if not isinstance(sys_modules, dict):
                        raise RuntimeError(
                            '\'sys.modules\' must be a dictionary not: {}'.format(
                                type(sys_modules)
                            )
                        )
                    patcher = patch.dict(sys.modules, sys_modules)
                    patcher.start()

                    def cleanup_sys_modules(patcher, sys_modules):
                        patcher.stop()
                        del patcher
                        del sys_modules

                    self.addCleanup(cleanup_sys_modules, patcher, sys_modules)

                for key in module_globals:
                    if not hasattr(module, key):
                        if key in salt_dunders:
                            setattr(module, key, {})
                        else:
                            setattr(module, key, None)

                if module_globals:
                    patcher = patch.multiple(module, **module_globals)
                    patcher.start()

                    def cleanup_module_globals(patcher, module_globals):
                        patcher.stop()
                        del patcher
                        del module_globals

                    self.addCleanup(cleanup_module_globals, patcher, module_globals)

                if minion_funcs:
                    # Since we autoloaded the minion_funcs, let's namespace the functions with the globals
                    # used to patch above
                    import salt.utils
                    for func in minion_funcs:
                        minion_funcs[func] = salt.utils.functools.namespaced_function(
                            minion_funcs[func],
                            module_globals,
                            preserve_context=True
                        )
            return setup_func(self)
        return wrapper

    def setup_loader_modules(self):
        raise NotImplementedError(
            '\'{}.setup_loader_modules()\' must be implemented'.format(self.__class__.__name__)
        )


class XMLEqualityMixin(object):

    def assertEqualXML(self, e1, e2):
        if six.PY3 and isinstance(e1, bytes):
            e1 = e1.decode('utf-8')
        if six.PY3 and isinstance(e2, bytes):
            e2 = e2.decode('utf-8')
        if isinstance(e1, six.string_types):
            e1 = etree.XML(e1)
        if isinstance(e2, six.string_types):
            e2 = etree.XML(e2)
        if e1.tag != e2.tag:
            return False
        if e1.text != e2.text:
            return False
        if e1.tail != e2.tail:
            return False
        if e1.attrib != e2.attrib:
            return False
        if len(e1) != len(e2):
            return False
        return all(self.assertEqualXML(c1, c2) for c1, c2 in zip(e1, e2))


class SaltReturnAssertsMixin(object):

    def assertReturnSaltType(self, ret):
        try:
            self.assertTrue(isinstance(ret, dict))
        except AssertionError:
            raise AssertionError(
                '{0} is not dict. Salt returned: {1}'.format(
                    type(ret).__name__, ret
                )
            )

    def assertReturnNonEmptySaltType(self, ret):
        self.assertReturnSaltType(ret)
        try:
            self.assertNotEqual(ret, {})
        except AssertionError:
            raise AssertionError(
                '{} is equal to {}. Salt returned an empty dictionary.'
            )

    def __return_valid_keys(self, keys):
        if isinstance(keys, tuple):
            # If it's a tuple, turn it into a list
            keys = list(keys)
        elif isinstance(keys, six.string_types):
            # If it's a string, make it a one item list
            keys = [keys]
        elif not isinstance(keys, list):
            # If we've reached here, it's a bad type passed to keys
            raise RuntimeError('The passed keys need to be a list')
        return keys

    def __getWithinSaltReturn(self, ret, keys):
        self.assertReturnNonEmptySaltType(ret)
        ret_data = []
        for part in six.itervalues(ret):
            keys = self.__return_valid_keys(keys)
            okeys = keys[:]
            try:
                ret_item = part[okeys.pop(0)]
            except (KeyError, TypeError):
                raise AssertionError(
                    'Could not get ret{0} from salt\'s return: {1}'.format(
                        ''.join(['[\'{0}\']'.format(k) for k in keys]), part
                    )
                )
            while okeys:
                try:
                    ret_item = ret_item[okeys.pop(0)]
                except (KeyError, TypeError):
                    raise AssertionError(
                        'Could not get ret{0} from salt\'s return: {1}'.format(
                            ''.join(['[\'{0}\']'.format(k) for k in keys]), part
                        )
                    )
            ret_data.append(ret_item)
        return ret_data

    def assertSaltTrueReturn(self, ret):
        try:
            for saltret in self.__getWithinSaltReturn(ret, 'result'):
                self.assertTrue(saltret)
        except AssertionError:
            log.info('Salt Full Return:\n{0}'.format(pprint.pformat(ret)))
            try:
                raise AssertionError(
                    '{result} is not True. Salt Comment:\n{comment}'.format(
                        **(next(six.itervalues(ret)))
                    )
                )
            except (AttributeError, IndexError):
                raise AssertionError(
                    'Failed to get result. Salt Returned:\n{0}'.format(
                        pprint.pformat(ret)
                    )
                )

    def assertSaltFalseReturn(self, ret):
        try:
            for saltret in self.__getWithinSaltReturn(ret, 'result'):
                self.assertFalse(saltret)
        except AssertionError:
            log.info('Salt Full Return:\n{0}'.format(pprint.pformat(ret)))
            try:
                raise AssertionError(
                    '{result} is not False. Salt Comment:\n{comment}'.format(
                        **(next(six.itervalues(ret)))
                    )
                )
            except (AttributeError, IndexError):
                raise AssertionError(
                    'Failed to get result. Salt Returned: {0}'.format(ret)
                )

    def assertSaltNoneReturn(self, ret):
        try:
            for saltret in self.__getWithinSaltReturn(ret, 'result'):
                self.assertIsNone(saltret)
        except AssertionError:
            log.info('Salt Full Return:\n{0}'.format(pprint.pformat(ret)))
            try:
                raise AssertionError(
                    '{result} is not None. Salt Comment:\n{comment}'.format(
                        **(next(six.itervalues(ret)))
                    )
                )
            except (AttributeError, IndexError):
                raise AssertionError(
                    'Failed to get result. Salt Returned: {0}'.format(ret)
                )

    def assertInSaltComment(self, in_comment, ret):
        for saltret in self.__getWithinSaltReturn(ret, 'comment'):
            self.assertIn(in_comment, saltret)

    def assertNotInSaltComment(self, not_in_comment, ret):
        for saltret in self.__getWithinSaltReturn(ret, 'comment'):
            self.assertNotIn(not_in_comment, saltret)

    def assertSaltCommentRegexpMatches(self, ret, pattern):
        return self.assertInSaltReturnRegexpMatches(ret, pattern, 'comment')

    def assertInSaltStateWarning(self, in_comment, ret):
        for saltret in self.__getWithinSaltReturn(ret, 'warnings'):
            self.assertIn(in_comment, saltret)

    def assertNotInSaltStateWarning(self, not_in_comment, ret):
        for saltret in self.__getWithinSaltReturn(ret, 'warnings'):
            self.assertNotIn(not_in_comment, saltret)

    def assertInSaltReturn(self, item_to_check, ret, keys):
        for saltret in self.__getWithinSaltReturn(ret, keys):
            self.assertIn(item_to_check, saltret)

    def assertNotInSaltReturn(self, item_to_check, ret, keys):
        for saltret in self.__getWithinSaltReturn(ret, keys):
            self.assertNotIn(item_to_check, saltret)

    def assertInSaltReturnRegexpMatches(self, ret, pattern, keys=()):
        for saltret in self.__getWithinSaltReturn(ret, keys):
            self.assertRegex(saltret, pattern)

    def assertSaltStateChangesEqual(self, ret, comparison, keys=()):
        keys = ['changes'] + self.__return_valid_keys(keys)
        for saltret in self.__getWithinSaltReturn(ret, keys):
            self.assertEqual(saltret, comparison)

    def assertSaltStateChangesNotEqual(self, ret, comparison, keys=()):
        keys = ['changes'] + self.__return_valid_keys(keys)
        for saltret in self.__getWithinSaltReturn(ret, keys):
            self.assertNotEqual(saltret, comparison)


def _fetch_events(q):
    '''
    Collect events and store them
    '''
    def _clean_queue():
        print('Cleaning queue!')
        while not q.empty():
            queue_item = q.get()
            queue_item.task_done()

    atexit.register(_clean_queue)
    a_config = AdaptedConfigurationTestCaseMixin()
    event = salt.utils.event.get_event('minion', sock_dir=a_config.get_config('minion')['sock_dir'], opts=a_config.get_config('minion'))
    while True:
        try:
            events = event.get_event(full=False)
        except Exception:
            # This is broad but we'll see all kinds of issues right now
            # if we drop the proc out from under the socket while we're reading
            pass
        q.put(events)


class SaltMinionEventAssertsMixin(object):
    '''
    Asserts to verify that a given event was seen
    '''

    def __new__(cls, *args, **kwargs):
        # We have to cross-call to re-gen a config
        cls.q = multiprocessing.Queue()
        cls.fetch_proc = salt.utils.process.SignalHandlingMultiprocessingProcess(
            target=_fetch_events, args=(cls.q,)
        )
        cls.fetch_proc.start()
        return object.__new__(cls)

    def __exit__(self, *args, **kwargs):
        self.fetch_proc.join()

    def assertMinionEventFired(self, tag):
        #TODO
        raise salt.exceptions.NotImplemented('assertMinionEventFired() not implemented')

    def assertMinionEventReceived(self, desired_event):
        queue_wait = 5  # 2.5s
        while self.q.empty():
            time.sleep(0.5)  # Wait for events to be pushed into the queue
            queue_wait -= 1
            if queue_wait <= 0:
                raise AssertionError('Queue wait timer expired')
        while not self.q.empty():  # This is not thread-safe and may be inaccurate
            event = self.q.get()
            if isinstance(event, dict):
                event.pop('_stamp')
            if desired_event == event:
                self.fetch_proc.terminate()
                return True
        self.fetch_proc.terminate()
        raise AssertionError('Event {0} was not received by minion'.format(desired_event))



# -*- coding: utf-8 -*-
'''
Helpers for testing man pages
'''
# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import sys

# Import Salt libs
import salt.utils.files
import salt.utils.path
import salt.utils.stringutils
from salt.exceptions import CommandExecutionError

# Import Salt Tesing libs
from tests.support.paths import CODE_DIR


def install(rootdir):
    if not os.path.exists(rootdir):
        os.makedirs(rootdir)
        return __salt__['cmd.retcode'](
            [sys.executable,
             os.path.join(CODE_DIR, 'setup.py'),
             'install', '--root={0}'.format(rootdir)]) == 0
    return True


def search(manpages, rootdir):
    manpage_fns = set(manpages)
    manpage_paths = {}
    for root, _, files in os.walk(rootdir):
        if not manpage_fns:
            # All manpages found, no need to keep walking
            break
        # Using list because we will be modifying the set during iteration
        for manpage_fn in list(manpage_fns):
            if manpage_fn in files:
                manpage_path = salt.utils.path.join(root, manpage_fn)
                manpage_paths[manpage_fn] = manpage_path
                manpage_fns.remove(manpage_fn)

    if manpage_fns:
        raise CommandExecutionError(
            'The following manpages were not found under {0}: {1}'.format(
                rootdir,
                ', '.join(sorted(manpage_fns))
            )
        )

    failed = {}
    for manpage in sorted(manpages):
        with salt.utils.files.fopen(manpage_paths[manpage]) as fp_:
            contents = salt.utils.stringutils.to_unicode(fp_.read())
        # Check for search string in contents
        for search_string in manpages[manpage]:
            if search_string not in contents:
                failed.setdefault(manpage, []).append(
                    'No match for search string \'{0}\' found in {1}'.format(
                        search_string, manpage_paths[manpage]
                    )
                )
        # Check for correct install dir
        path = '/man{0}/'.format(manpage.rsplit('.', 1)[-1])
        if path not in manpage_paths[manpage]:
            failed.setdefault(manpage, []).append(
                '{0} not found in manpage path {1}'.format(
                    path, manpage_paths[manpage]
                )
            )

    if failed:
        raise CommandExecutionError('One or more manpages failed', info=failed)

    return True



# -*- coding: utf-8 -*-
'''
Venafi Pillar Certificates

This module will only return pillar data if the ``venafi`` runner module has
already been used to create certificates.

To configure this module, set ``venafi`` to ``True`` in the ``ext_pillar``
section of your ``master`` configuration file:

.. code-block:: yaml

    ext_pillar:
      - venafi: True
'''
from __future__ import absolute_import, print_function, unicode_literals
import logging
import salt.cache
import salt.syspaths as syspaths

__virtualname__ = 'venafi'
log = logging.getLogger(__name__)


def __virtual__():
    '''
    No special requirements outside of Salt itself
    '''
    return __virtualname__


def ext_pillar(minion_id, pillar, conf):
    '''
    Return an existing set of certificates
    '''
    cache = salt.cache.Cache(__opts__, syspaths.CACHE_DIR)

    ret = {}
    dns_names = cache.fetch('venafi/minions', minion_id)
    for dns_name in dns_names:
        data = cache.fetch('venafi/domains', dns_name)
        ret[dns_name] = data
        del ret[dns_name]['csr']
    return {'venafi': ret}



# encoding: utf-8

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals
import os
import time

# Import Salt Testing libs
from tests.support.paths import TMP_CONF_DIR
from tests.support.unit import TestCase, skipIf

# Import Salt libs
import salt.config
import salt.netapi


class NetapiClientTest(TestCase):
    eauth_creds = {
        'username': 'saltdev_auto',
        'password': 'saltdev',
        'eauth': 'auto',
    }

    def setUp(self):
        '''
        Set up a NetapiClient instance
        '''
        opts = salt.config.client_config(os.path.join(TMP_CONF_DIR, 'master'))
        self.netapi = salt.netapi.NetapiClient(opts)

    def tearDown(self):
        del self.netapi

    def test_local(self):
        low = {'client': 'local', 'tgt': '*', 'fun': 'test.ping'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        self.assertEqual(ret, {'minion': True, 'sub_minion': True})

    def test_local_batch(self):
        low = {'client': 'local_batch', 'tgt': '*', 'fun': 'test.ping'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        rets = []
        for _ret in ret:
            rets.append(_ret)
        self.assertIn({'sub_minion': True}, rets)
        self.assertIn({'minion': True}, rets)

    def test_local_async(self):
        low = {'client': 'local_async', 'tgt': '*', 'fun': 'test.ping'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

        # Remove all the volatile values before doing the compare.
        self.assertIn('jid', ret)
        ret.pop('jid', None)
        ret['minions'] = sorted(ret['minions'])
        self.assertEqual(ret, {'minions': sorted(['minion', 'sub_minion'])})

    def test_wheel(self):
        low = {'client': 'wheel', 'fun': 'key.list_all'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

        # Remove all the volatile values before doing the compare.
        self.assertIn('tag', ret)
        ret.pop('tag')

        data = ret.get('data', {})
        self.assertIn('jid', data)
        data.pop('jid', None)

        self.assertIn('tag', data)
        data.pop('tag', None)

        ret.pop('_stamp', None)
        data.pop('_stamp', None)

        self.maxDiff = None
        self.assertTrue(set(['master.pem', 'master.pub']).issubset(set(ret['data']['return']['local'])))

    def test_wheel_async(self):
        # Give this test a little breathing room
        time.sleep(3)
        low = {'client': 'wheel_async', 'fun': 'key.list_all'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)
        self.assertIn('jid', ret)
        self.assertIn('tag', ret)

    @skipIf(True, 'This is not testing anything. Skipping for now.')
    def test_runner(self):
        # TODO: fix race condition in init of event-- right now the event class
        # will finish init even if the underlying zmq socket hasn't connected yet
        # this is problematic for the runnerclient's master_call method if the
        # runner is quick
        #low = {'client': 'runner', 'fun': 'cache.grains'}
        low = {'client': 'runner', 'fun': 'test.sleep', 'arg': [2]}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)

    @skipIf(True, 'This is not testing anything. Skipping for now.')
    def test_runner_async(self):
        low = {'client': 'runner', 'fun': 'cache.grains'}
        low.update(self.eauth_creds)

        ret = self.netapi.run(low)



# -*- coding: utf-8 -*-
'''
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
'''
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import skipIf, TestCase
from tests.support.mock import (
    NO_MOCK,
    NO_MOCK_REASON,
    MagicMock,
    patch)

# Import Salt Libs
import salt.states.at as at


@skipIf(NO_MOCK, NO_MOCK_REASON)
class AtTestCase(TestCase, LoaderModuleMockMixin):
    '''
    Test cases for salt.states.at
    '''
    def setup_loader_modules(self):
        return {at: {}}

    # 'present' function tests: 1

    def test_present(self):
        '''
        Test to add a job to queue.
        '''

        # input variables
        name = 'jboss'
        timespec = '9:09 11/04/15'
        tag = 'love'
        user = 'jam'

        # mock for at.at module call
        mock_atat = {
            'jobs': [
                {
                    'date': '2015-11-04',
                    'job': '1476031633.a',
                    'queue': 'a',
                    'tag': tag,
                    'time': '09:09:00',
                    'user': user,
                },
            ],
        }

        # normale return
        ret = {
            'name': name,
            'result': True,
            'changes': {
                'date': '2015-11-04',
                'job': '1476031633.a',
                'queue': 'a',
                'tag': 'love',
                'time': '09:09:00',
                'user': 'jam',
            },
            'comment': 'job {name} added and will run on {timespec}'.format(
                name=name,
                timespec=timespec
            ),
        }

        # unknown user return
        ret_user = {}
        ret_user.update(ret)
        ret_user.update({
            'result': False,
            'changes': {},
            'comment': 'user {0} does not exists'.format(user),
        })

        # add a job
        mock = MagicMock(return_value=mock_atat)
        with patch.dict(at.__opts__, {'test': False}):
            with patch.dict(at.__salt__, {'at.at': mock}):
                self.assertDictEqual(at.present(name, timespec, tag), ret)

        # add a job with a non-existing user
        mock = MagicMock(return_value=False)
        with patch.dict(at.__opts__, {'test': False}):
            with patch.dict(at.__salt__, {'user.info': mock}):
                self.assertDictEqual(at.present(name, timespec, tag, user), ret_user)

        # add a job with test=True
        with patch.dict(at.__opts__, {'test': True}):
            ret_test = {}
            ret_test.update(ret)
            ret_test.update({'result': None, 'changes': {}})
            self.assertDictEqual(at.present(name, timespec, tag, user), ret_test)

    # 'absent' function tests: 1

    def test_absent(self):
        '''
        Test to remove a job from queue
        '''
        # input variables
        name = 'jboss'
        tag = 'rose'
        user = 'jam'

        # mock for at.atrm module call
        mock_atatrm = {
            'jobs': {
                'removed': ['1476033859.a', '1476033855.a'],
                'tag': None,
             },
        }

        # mock for at.jobcheck module call
        mock_atjobcheck = {
            'jobs': [
                {
                    'date': '2015-11-04',
                    'job': '1476031633.a',
                    'queue': 'a',
                    'tag': tag,
                    'time': '09:09:00',
                    'user': user,
                },
            ],
        }

        # normal return
        ret = {
            'name': name,
            'result': True,
            'changes': {
                'removed': ['1476033859.a', '1476033855.a'],
             },
            'comment': 'removed 2 job(s)',
        }

        # remove a job with test=True
        with patch.dict(at.__opts__, {'test': True}):
            ret_test = {}
            ret_test.update(ret)
            ret_test.update({
                'result': None,
                'changes': {},
                'comment': 'removed ? job(s)'
            })
            self.assertDictEqual(at.absent(name), ret_test)

        # remove a job and pass limit parameter
        with patch.dict(at.__opts__, {'test': False}):
            ret_limit = {}
            ret_limit.update(ret)
            ret_limit.update({
                'result': False,
                'changes': {},
                'comment': 'limit parameter not supported {0}'.format(name),
            })
            self.assertDictEqual(at.absent(name, limit='all'), ret_limit)

        # remove all jobs (2 jobs found)
        mock = MagicMock(return_value=mock_atatrm)
        with patch.dict(at.__salt__, {'at.atrm': mock}):
            with patch.dict(at.__opts__, {'test': False}):
                self.assertDictEqual(at.absent(name), ret)

        # remove all jobs (0 jobs found)
        mock_atatrm_nojobs = {}
        mock_atatrm_nojobs.update(mock_atatrm)
        mock_atatrm_nojobs.update({
            'jobs': {
                'removed': [],
            },
        })
        mock = MagicMock(return_value=mock_atatrm_nojobs)
        with patch.dict(at.__salt__, {'at.atrm': mock}):
            with patch.dict(at.__opts__, {'test': False}):
                ret_nojobs = {}
                ret_nojobs.update(ret)
                ret_nojobs.update({
                    'changes': {},
                    'comment': ret['comment'].replace('2', '0'),
                })
                self.assertDictEqual(at.absent(name), ret_nojobs)

        # remove all tagged jobs (1 jobs found)
        mock_atatrm_tag = {}
        mock_atatrm_tag.update(mock_atatrm)
        mock_atatrm_tag.update({
            'jobs': {
                'removed': ['1476031633.a'],
                'tag': 'rose',
            },
        })
        mock = MagicMock(return_value=mock_atatrm_tag)
        with patch.dict(at.__salt__, {'at.atrm': mock}):
            mock = MagicMock(return_value=mock_atjobcheck)
            with patch.dict(at.__salt__, {'at.jobcheck': mock}):
                with patch.dict(at.__opts__, {'test': False}):
                    ret_tag = {}
                    ret_tag.update(ret)
                    ret_tag.update({
                        'changes': {
                            'removed': ['1476031633.a'],
                        },
                        'comment': ret['comment'].replace('2', '1'),
                    })
                    self.assertDictEqual(at.absent(name, tag=tag), ret_tag)



# -*- coding: utf-8 -*-
'''
    :codeauthor: :email:`Alexandru Bleotu <alexandru.bleotu@morganstanley.com>`

    Tests functions in salt.utils.vsan
'''

# Import python libraries
from __future__ import absolute_import, print_function, unicode_literals
import logging

# Import Salt testing libraries
from tests.support.unit import TestCase, skipIf
from tests.support.mock import NO_MOCK, NO_MOCK_REASON, patch, MagicMock, \
        PropertyMock

# Import Salt libraries
from salt.exceptions import VMwareApiError, VMwareRuntimeError, \
        VMwareObjectRetrievalError
from salt.ext.six.moves import range
import salt.utils.pbm

try:
    from pyVmomi import vim, vmodl, pbm
    HAS_PYVMOMI = True
except ImportError:
    HAS_PYVMOMI = False


# Get Logging Started
log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetProfileManagerTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_profile_manager'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_stub = MagicMock()
        self.mock_prof_mgr = MagicMock()
        self.mock_content = MagicMock()
        self.mock_pbm_si = MagicMock(
            RetrieveContent=MagicMock(return_value=self.mock_content))
        type(self.mock_content).profileManager = \
                PropertyMock(return_value=self.mock_prof_mgr)
        patches = (
            ('salt.utils.vmware.get_new_service_instance_stub',
             MagicMock(return_value=self.mock_stub)),
            ('salt.utils.pbm.pbm.ServiceInstance',
             MagicMock(return_value=self.mock_pbm_si)))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_stub', 'mock_content',
                     'mock_pbm_si', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_get_new_service_stub(self):
        mock_get_new_service_stub = MagicMock()
        with patch('salt.utils.vmware.get_new_service_instance_stub',
                   mock_get_new_service_stub):
            salt.utils.pbm.get_profile_manager(self.mock_si)
        mock_get_new_service_stub.assert_called_once_with(
            self.mock_si, ns='pbm/2.0', path='/pbm/sdk')

    def test_pbm_si(self):
        mock_get_pbm_si = MagicMock()
        with patch('salt.utils.pbm.pbm.ServiceInstance',
                   mock_get_pbm_si):
            salt.utils.pbm.get_profile_manager(self.mock_si)
        mock_get_pbm_si.assert_called_once_with('ServiceInstance',
                                                self.mock_stub)

    def test_return_profile_manager(self):
        ret = salt.utils.pbm.get_profile_manager(self.mock_si)
        self.assertEqual(ret, self.mock_prof_mgr)

    def test_profile_manager_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        type(self.mock_content).profileManager = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_profile_manager(self.mock_si)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_profile_manager_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        type(self.mock_content).profileManager = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_profile_manager(self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_profile_manager_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        type(self.mock_content).profileManager = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_profile_manager(self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetPlacementSolverTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_placement_solver'''
    def setUp(self):
        self.mock_si = MagicMock()
        self.mock_stub = MagicMock()
        self.mock_prof_mgr = MagicMock()
        self.mock_content = MagicMock()
        self.mock_pbm_si = MagicMock(
            RetrieveContent=MagicMock(return_value=self.mock_content))
        type(self.mock_content).placementSolver = \
                PropertyMock(return_value=self.mock_prof_mgr)
        patches = (
            ('salt.utils.vmware.get_new_service_instance_stub',
             MagicMock(return_value=self.mock_stub)),
            ('salt.utils.pbm.pbm.ServiceInstance',
             MagicMock(return_value=self.mock_pbm_si)))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_si', 'mock_stub', 'mock_content',
                     'mock_pbm_si', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_get_new_service_stub(self):
        mock_get_new_service_stub = MagicMock()
        with patch('salt.utils.vmware.get_new_service_instance_stub',
                   mock_get_new_service_stub):
            salt.utils.pbm.get_placement_solver(self.mock_si)
        mock_get_new_service_stub.assert_called_once_with(
            self.mock_si, ns='pbm/2.0', path='/pbm/sdk')

    def test_pbm_si(self):
        mock_get_pbm_si = MagicMock()
        with patch('salt.utils.pbm.pbm.ServiceInstance',
                   mock_get_pbm_si):
            salt.utils.pbm.get_placement_solver(self.mock_si)
        mock_get_pbm_si.assert_called_once_with('ServiceInstance',
                                                self.mock_stub)

    def test_return_profile_manager(self):
        ret = salt.utils.pbm.get_placement_solver(self.mock_si)
        self.assertEqual(ret, self.mock_prof_mgr)

    def test_placement_solver_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        type(self.mock_content).placementSolver = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_placement_solver(self.mock_si)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_placement_solver_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        type(self.mock_content).placementSolver = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_placement_solver(self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_placement_solver_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        type(self.mock_content).placementSolver = PropertyMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_placement_solver(self.mock_si)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetCapabilityDefinitionsTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_capability_definitions'''
    def setUp(self):
        self.mock_res_type = MagicMock()
        self.mock_cap_cats = [MagicMock(capabilityMetadata=['fake_cap_meta1',
                                                            'fake_cap_meta2']),
                              MagicMock(capabilityMetadata=['fake_cap_meta3'])]
        self.mock_prof_mgr = MagicMock(
            FetchCapabilityMetadata=MagicMock(return_value=self.mock_cap_cats))
        patches = (
            ('salt.utils.pbm.pbm.profile.ResourceType',
             MagicMock(return_value=self.mock_res_type)),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_res_type', 'mock_cap_cats', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_get_res_type(self):
        mock_get_res_type = MagicMock()
        with patch('salt.utils.pbm.pbm.profile.ResourceType',
                   mock_get_res_type):
            salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        mock_get_res_type.assert_called_once_with(
            resourceType=pbm.profile.ResourceTypeEnum.STORAGE)

    def test_fetch_capabilities(self):
        salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        self.mock_prof_mgr.FetchCapabilityMetadata.assert_called_once_with(
            self.mock_res_type)

    def test_fetch_capabilities_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.FetchCapabilityMetadata = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_fetch_capabilities_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.FetchCapabilityMetadata = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_fetch_capabilities_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.FetchCapabilityMetadata = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')

    def test_return_cap_definitions(self):
        ret = salt.utils.pbm.get_capability_definitions(self.mock_prof_mgr)
        self.assertEqual(ret, ['fake_cap_meta1', 'fake_cap_meta2',
                               'fake_cap_meta3'])


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetPoliciesByIdTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_policies_by_id'''
    def setUp(self):
        self.policy_ids = MagicMock()
        self.mock_policies = MagicMock()
        self.mock_prof_mgr = MagicMock(
            RetrieveContent=MagicMock(return_value=self.mock_policies))

    def tearDown(self):
        for attr in ('policy_ids', 'mock_policies', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_retrieve_policies(self):
        salt.utils.pbm.get_policies_by_id(self.mock_prof_mgr, self.policy_ids)
        self.mock_prof_mgr.RetrieveContent.assert_called_once_with(
            self.policy_ids)

    def test_retrieve_policies_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.RetrieveContent = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_policies_by_id(self.mock_prof_mgr, self.policy_ids)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_retrieve_policies_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.RetrieveContent = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_policies_by_id(self.mock_prof_mgr, self.policy_ids)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_retrieve_policies_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.RetrieveContent = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_policies_by_id(self.mock_prof_mgr, self.policy_ids)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')

    def test_return_policies(self):
        ret = salt.utils.pbm.get_policies_by_id(self.mock_prof_mgr, self.policy_ids)
        self.assertEqual(ret, self.mock_policies)


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetStoragePoliciesTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_storage_policies'''
    def setUp(self):
        self.mock_res_type = MagicMock()
        self.mock_policy_ids = MagicMock()
        self.mock_prof_mgr = MagicMock(
            QueryProfile=MagicMock(return_value=self.mock_policy_ids))
        # Policies
        self.mock_policies = []
        for i in range(4):
            mock_obj = MagicMock(resourceType=MagicMock(
                resourceType=pbm.profile.ResourceTypeEnum.STORAGE))
            mock_obj.name = 'fake_policy{0}'.format(i)
            self.mock_policies.append(mock_obj)
        patches = (
            ('salt.utils.pbm.pbm.profile.ResourceType',
             MagicMock(return_value=self.mock_res_type)),
            ('salt.utils.pbm.get_policies_by_id',
             MagicMock(return_value=self.mock_policies)))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_res_type', 'mock_policy_ids', 'mock_policies',
                     'mock_prof_mgr'):
            delattr(self, attr)

    def test_get_res_type(self):
        mock_get_res_type = MagicMock()
        with patch('salt.utils.pbm.pbm.profile.ResourceType',
                   mock_get_res_type):
            salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        mock_get_res_type.assert_called_once_with(
            resourceType=pbm.profile.ResourceTypeEnum.STORAGE)

    def test_retrieve_policy_ids(self):
        mock_retrieve_policy_ids = MagicMock(return_value=self.mock_policy_ids)
        self.mock_prof_mgr.QueryProfile = mock_retrieve_policy_ids
        salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        mock_retrieve_policy_ids.assert_called_once_with(self.mock_res_type)

    def test_retrieve_policy_ids_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.QueryProfile = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_retrieve_policy_ids_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.QueryProfile = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_retrieve_policy_ids_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.QueryProfile = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')

    def test_get_policies_by_id(self):
        mock_get_policies_by_id = MagicMock(return_value=self.mock_policies)
        with patch('salt.utils.pbm.get_policies_by_id',
                   mock_get_policies_by_id):
            salt.utils.pbm.get_storage_policies(self.mock_prof_mgr)
        mock_get_policies_by_id.assert_called_once_with(
            self.mock_prof_mgr, self.mock_policy_ids)

    def test_return_all_policies(self):
        ret = salt.utils.pbm.get_storage_policies(self.mock_prof_mgr,
                                                  get_all_policies=True)
        self.assertEqual(ret, self.mock_policies)

    def test_return_filtered_policies(self):
        ret = salt.utils.pbm.get_storage_policies(
            self.mock_prof_mgr, policy_names=['fake_policy1', 'fake_policy3'])
        self.assertEqual(ret, [self.mock_policies[1], self.mock_policies[3]])


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class CreateStoragePolicyTestCase(TestCase):
    '''Tests for salt.utils.pbm.create_storage_policy'''
    def setUp(self):
        self.mock_policy_spec = MagicMock()
        self.mock_prof_mgr = MagicMock()

    def tearDown(self):
        for attr in ('mock_policy_spec', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_create_policy(self):
        salt.utils.pbm.create_storage_policy(self.mock_prof_mgr,
                                             self.mock_policy_spec)
        self.mock_prof_mgr.Create.assert_called_once_with(
            self.mock_policy_spec)

    def test_create_policy_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.Create = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.create_storage_policy(self.mock_prof_mgr,
                                                 self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_create_policy_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.Create = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.create_storage_policy(self.mock_prof_mgr,
                                                 self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_create_policy_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.Create = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.create_storage_policy(self.mock_prof_mgr,
                                                 self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class UpdateStoragePolicyTestCase(TestCase):
    '''Tests for salt.utils.pbm.update_storage_policy'''
    def setUp(self):
        self.mock_policy_spec = MagicMock()
        self.mock_policy = MagicMock()
        self.mock_prof_mgr = MagicMock()

    def tearDown(self):
        for attr in ('mock_policy_spec', 'mock_policy', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_create_policy(self):
        salt.utils.pbm.update_storage_policy(
            self.mock_prof_mgr, self.mock_policy, self.mock_policy_spec)
        self.mock_prof_mgr.Update.assert_called_once_with(
            self.mock_policy.profileId, self.mock_policy_spec)

    def test_create_policy_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.Update = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.update_storage_policy(
                self.mock_prof_mgr, self.mock_policy, self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_create_policy_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.Update = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.update_storage_policy(
                self.mock_prof_mgr, self.mock_policy, self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_create_policy_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.Update = MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.update_storage_policy(
                self.mock_prof_mgr, self.mock_policy, self.mock_policy_spec)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class GetDefaultStoragePolicyOfDatastoreTestCase(TestCase):
    '''Tests for salt.utils.pbm.get_default_storage_policy_of_datastore'''
    def setUp(self):
        self.mock_ds = MagicMock(_moId='fake_ds_moid')
        self.mock_hub = MagicMock()
        self.mock_policy_id = 'fake_policy_id'
        self.mock_prof_mgr = MagicMock(
            QueryDefaultRequirementProfile=MagicMock(
                return_value=self.mock_policy_id))
        self.mock_policy_refs = [MagicMock()]
        patches = (
            ('salt.utils.pbm.pbm.placement.PlacementHub',
             MagicMock(return_value=self.mock_hub)),
            ('salt.utils.pbm.get_policies_by_id',
             MagicMock(return_value=self.mock_policy_refs)))
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_ds', 'mock_hub', 'mock_policy_id', 'mock_prof_mgr',
                     'mock_policy_refs'):
            delattr(self, attr)

    def test_get_placement_hub(self):
        mock_get_placement_hub = MagicMock()
        with patch('salt.utils.pbm.pbm.placement.PlacementHub',
                   mock_get_placement_hub):
            salt.utils.pbm.get_default_storage_policy_of_datastore(
                self.mock_prof_mgr, self.mock_ds)
        mock_get_placement_hub.assert_called_once_with(
            hubId='fake_ds_moid', hubType='Datastore')

    def test_query_default_requirement_profile(self):
        mock_query_prof = MagicMock(return_value=self.mock_policy_id)
        self.mock_prof_mgr.QueryDefaultRequirementProfile = \
                mock_query_prof
        salt.utils.pbm.get_default_storage_policy_of_datastore(
            self.mock_prof_mgr, self.mock_ds)
        mock_query_prof.assert_called_once_with(self.mock_hub)

    def test_query_default_requirement_profile_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.QueryDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_default_storage_policy_of_datastore(
                self.mock_prof_mgr, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_query_default_requirement_profile_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.QueryDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.get_default_storage_policy_of_datastore(
                self.mock_prof_mgr, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_query_default_requirement_profile_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.QueryDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.get_default_storage_policy_of_datastore(
                self.mock_prof_mgr, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')

    def test_get_policies_by_id(self):
        mock_get_policies_by_id = MagicMock()
        with patch('salt.utils.pbm.get_policies_by_id',
                   mock_get_policies_by_id):
            salt.utils.pbm.get_default_storage_policy_of_datastore(
                self.mock_prof_mgr, self.mock_ds)
        mock_get_policies_by_id.assert_called_once_with(
            self.mock_prof_mgr, [self.mock_policy_id])

    def test_no_policy_refs(self):
        mock_get_policies_by_id = MagicMock()
        with patch('salt.utils.pbm.get_policies_by_id',
                  MagicMock(return_value=None)):
            with self.assertRaises(VMwareObjectRetrievalError) as excinfo:
                salt.utils.pbm.get_default_storage_policy_of_datastore(
                    self.mock_prof_mgr, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror,
                         'Storage policy with id \'fake_policy_id\' was not '
                         'found')

    def test_return_policy_ref(self):
        mock_get_policies_by_id = MagicMock()
        ret = salt.utils.pbm.get_default_storage_policy_of_datastore(
            self.mock_prof_mgr, self.mock_ds)
        self.assertEqual(ret, self.mock_policy_refs[0])


@skipIf(NO_MOCK, NO_MOCK_REASON)
@skipIf(not HAS_PYVMOMI, 'The \'pyvmomi\' library is missing')
class AssignDefaultStoragePolicyToDatastoreTestCase(TestCase):
    '''Tests for salt.utils.pbm.assign_default_storage_policy_to_datastore'''
    def setUp(self):
        self.mock_ds = MagicMock(_moId='fake_ds_moid')
        self.mock_policy = MagicMock()
        self.mock_hub = MagicMock()
        self.mock_prof_mgr = MagicMock()
        patches = (
            ('salt.utils.pbm.pbm.placement.PlacementHub',
             MagicMock(return_value=self.mock_hub)),)
        for mod, mock in patches:
            patcher = patch(mod, mock)
            patcher.start()
            self.addCleanup(patcher.stop)

    def tearDown(self):
        for attr in ('mock_ds', 'mock_hub', 'mock_policy', 'mock_prof_mgr'):
            delattr(self, attr)

    def test_get_placement_hub(self):
        mock_get_placement_hub = MagicMock()
        with patch('salt.utils.pbm.pbm.placement.PlacementHub',
                   mock_get_placement_hub):
            salt.utils.pbm.assign_default_storage_policy_to_datastore(
                self.mock_prof_mgr, self.mock_policy, self.mock_ds)
        mock_get_placement_hub.assert_called_once_with(
            hubId='fake_ds_moid', hubType='Datastore')

    def test_assign_default_requirement_profile(self):
        mock_assign_prof = MagicMock()
        self.mock_prof_mgr.AssignDefaultRequirementProfile = \
                mock_assign_prof
        salt.utils.pbm.assign_default_storage_policy_to_datastore(
            self.mock_prof_mgr, self.mock_policy, self.mock_ds)
        mock_assign_prof.assert_called_once_with(
            self.mock_policy.profileId, [self.mock_hub])

    def test_assign_default_requirement_profile_raises_no_permissions(self):
        exc = vim.fault.NoPermission()
        exc.privilegeId = 'Fake privilege'
        self.mock_prof_mgr.AssignDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.assign_default_storage_policy_to_datastore(
                self.mock_prof_mgr, self.mock_policy, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror,
                         'Not enough permissions. Required privilege: '
                         'Fake privilege')

    def test_assign_default_requirement_profile_raises_vim_fault(self):
        exc = vim.fault.VimFault()
        exc.msg = 'VimFault msg'
        self.mock_prof_mgr.AssignDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareApiError) as excinfo:
            salt.utils.pbm.assign_default_storage_policy_to_datastore(
                self.mock_prof_mgr, self.mock_policy, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror, 'VimFault msg')

    def test_assign_default_requirement_profile_raises_runtime_fault(self):
        exc = vmodl.RuntimeFault()
        exc.msg = 'RuntimeFault msg'
        self.mock_prof_mgr.AssignDefaultRequirementProfile = \
                MagicMock(side_effect=exc)
        with self.assertRaises(VMwareRuntimeError) as excinfo:
            salt.utils.pbm.assign_default_storage_policy_to_datastore(
                self.mock_prof_mgr, self.mock_policy, self.mock_ds)
        self.assertEqual(excinfo.exception.strerror, 'RuntimeFault msg')



# -*- coding: utf-8 -*-
'''
    :codeauthor: Pedro Algarvio (pedro@algarvio.me)


    tests.unit.payload_test
    ~~~~~~~~~~~~~~~~~~~~~~~
'''

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import time
import errno
import threading
import datetime

# Import Salt Testing libs
from tests.support.unit import skipIf, TestCase
from tests.support.mock import NO_MOCK, NO_MOCK_REASON

# Import Salt libs
from salt.utils import immutabletypes
from salt.utils.odict import OrderedDict
import salt.exceptions
import salt.payload

# Import 3rd-party libs
import zmq
from salt.ext import six

import logging

log = logging.getLogger(__name__)


@skipIf(NO_MOCK, NO_MOCK_REASON)
class PayloadTestCase(TestCase):

    def assertNoOrderedDict(self, data):
        if isinstance(data, OrderedDict):
            raise AssertionError(
                'Found an ordered dictionary'
            )
        if isinstance(data, dict):
            for value in six.itervalues(data):
                self.assertNoOrderedDict(value)
        elif isinstance(data, (list, tuple)):
            for chunk in data:
                self.assertNoOrderedDict(chunk)

    def test_list_nested_odicts(self):
        payload = salt.payload.Serial('msgpack')
        idata = {'pillar': [OrderedDict(environment='dev')]}
        odata = payload.loads(payload.dumps(idata.copy()))
        self.assertNoOrderedDict(odata)
        self.assertEqual(idata, odata)

    def test_datetime_dump_load(self):
        '''
        Check the custom datetime handler can understand itself
        '''
        payload = salt.payload.Serial('msgpack')
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        idata = {dtvalue: dtvalue}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata)
        self.assertEqual(
                sdata,
                b'\x81\xc7\x18N20010203T04:05:06.000007\xc7\x18N20010203T04:05:06.000007')
        self.assertEqual(idata, odata)

    def test_verylong_dump_load(self):
        '''
        Test verylong encoder/decoder
        '''
        payload = salt.payload.Serial('msgpack')
        idata = {'jid': 20180227140750302662}
        sdata = payload.dumps(idata.copy())
        odata = payload.loads(sdata)
        idata['jid'] = '{0}'.format(idata['jid'])
        self.assertEqual(idata, odata)

    def test_immutable_dict_dump_load(self):
        '''
        Test immutable dict encoder/decoder
        '''
        payload = salt.payload.Serial('msgpack')
        idata = {'dict': {'key': 'value'}}
        sdata = payload.dumps({'dict': immutabletypes.ImmutableDict(idata['dict'])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_immutable_list_dump_load(self):
        '''
        Test immutable list encoder/decoder
        '''
        payload = salt.payload.Serial('msgpack')
        idata = {'list': [1, 2, 3]}
        sdata = payload.dumps({'list': immutabletypes.ImmutableList(idata['list'])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_immutable_set_dump_load(self):
        '''
        Test immutable set encoder/decoder
        '''
        payload = salt.payload.Serial('msgpack')
        idata = {'set': ['red', 'green', 'blue']}
        sdata = payload.dumps({'set': immutabletypes.ImmutableSet(idata['set'])})
        odata = payload.loads(sdata)
        self.assertEqual(idata, odata)

    def test_odict_dump_load(self):
        '''
        Test odict just works. It wasn't until msgpack 0.2.0
        '''
        payload = salt.payload.Serial('msgpack')
        data = OrderedDict()
        data['a'] = 'b'
        data['y'] = 'z'
        data['j'] = 'k'
        data['w'] = 'x'
        sdata = payload.dumps({'set': data})
        odata = payload.loads(sdata)
        self.assertEqual({'set': dict(data)}, odata)

    def test_mixed_dump_load(self):
        '''
        Test we can handle all exceptions at once
        '''
        payload = salt.payload.Serial('msgpack')
        dtvalue = datetime.datetime(2001, 2, 3, 4, 5, 6, 7)
        od = OrderedDict()
        od['a'] = 'b'
        od['y'] = 'z'
        od['j'] = 'k'
        od['w'] = 'x'
        idata = {dtvalue: dtvalue,  # datetime
                 'jid': 20180227140750302662,  # long int
                 'dict': immutabletypes.ImmutableDict({'key': 'value'}),  # immutable dict
                 'list': immutabletypes.ImmutableList([1, 2, 3]),  # immutable list
                 'set': immutabletypes.ImmutableSet(('red', 'green', 'blue')),  # immutable set
                 'odict': od,  # odict
                 }
        edata = {dtvalue: dtvalue,  # datetime, == input
                 'jid': '20180227140750302662',  # string repr of long int
                 'dict': {'key': 'value'},  # builtin dict
                 'list': [1, 2, 3],  # builtin list
                 'set': ['red', 'green', 'blue'],  # builtin set
                 'odict': dict(od),  # builtin dict
                 }
        sdata = payload.dumps(idata)
        odata = payload.loads(sdata)
        self.assertEqual(edata, odata)


class SREQTestCase(TestCase):
    port = 8845  # TODO: dynamically assign a port?

    @classmethod
    def setUpClass(cls):
        '''
        Class to set up zmq echo socket
        '''
        def echo_server():
            '''
            A server that echos the message sent to it over zmq

            Optional "sleep" can be sent to delay response
            '''
            context = zmq.Context()
            socket = context.socket(zmq.REP)
            socket.bind("tcp://*:{0}".format(SREQTestCase.port))
            payload = salt.payload.Serial('msgpack')

            while SREQTestCase.thread_running.is_set():
                try:
                    #  Wait for next request from client
                    message = socket.recv(zmq.NOBLOCK)
                    msg_deserialized = payload.loads(message)
                    log.info('Echo server received message: %s', msg_deserialized)
                    if isinstance(msg_deserialized['load'], dict) and msg_deserialized['load'].get('sleep'):
                        log.info('Test echo server sleeping for %s seconds',
                                 msg_deserialized['load']['sleep'])
                        time.sleep(msg_deserialized['load']['sleep'])
                    socket.send(message)
                except zmq.ZMQError as exc:
                    if exc.errno == errno.EAGAIN:
                        continue
                    raise
        SREQTestCase.thread_running = threading.Event()
        SREQTestCase.thread_running.set()
        SREQTestCase.echo_server = threading.Thread(target=echo_server)
        SREQTestCase.echo_server.start()

    @classmethod
    def tearDownClass(cls):
        '''
        Remove echo server
        '''
        # kill the thread
        SREQTestCase.thread_running.clear()
        SREQTestCase.echo_server.join()

    def get_sreq(self):
        return salt.payload.SREQ('tcp://127.0.0.1:{0}'.format(SREQTestCase.port))

    def test_send_auto(self):
        '''
        Test creation, send/rect
        '''
        sreq = self.get_sreq()
        # check default of empty load and enc clear
        assert sreq.send_auto({}) == {'enc': 'clear', 'load': {}}

        # check that the load always gets passed
        assert sreq.send_auto({'load': 'foo'}) == {'load': 'foo', 'enc': 'clear'}

    def test_send(self):
        sreq = self.get_sreq()
        assert sreq.send('clear', 'foo') == {'enc': 'clear', 'load': 'foo'}

    @skipIf(True, 'Disabled until we can figure out how to make this more reliable.')
    def test_timeout(self):
        '''
        Test SREQ Timeouts
        '''
        sreq = self.get_sreq()
        # client-side timeout
        start = time.time()
        # This is a try/except instead of an assertRaises because of a possible
        # subtle bug in zmq wherein a timeout=0 actually exceutes a single poll
        # before the timeout is reached.
        log.info('Sending tries=0, timeout=0')
        try:
            sreq.send('clear', 'foo', tries=0, timeout=0)
        except salt.exceptions.SaltReqTimeoutError:
            pass
        assert time.time() - start < 1  # ensure we didn't wait

        # server-side timeout
        log.info('Sending tries=1, timeout=1')
        start = time.time()
        with self.assertRaises(salt.exceptions.SaltReqTimeoutError):
            sreq.send('clear', {'sleep': 2}, tries=1, timeout=1)
        assert time.time() - start >= 1  # ensure we actually tried once (1s)

        # server-side timeout with retries
        log.info('Sending tries=2, timeout=1')
        start = time.time()
        with self.assertRaises(salt.exceptions.SaltReqTimeoutError):
            sreq.send('clear', {'sleep': 2}, tries=2, timeout=1)
        assert time.time() - start >= 2  # ensure we actually tried twice (2s)

        # test a regular send afterwards (to make sure sockets aren't in a twist
        log.info('Sending regular send')
        assert sreq.send('clear', 'foo') == {'enc': 'clear', 'load': 'foo'}

    def test_destroy(self):
        '''
        Test the __del__ capabilities
        '''
        sreq = self.get_sreq()
        # ensure no exceptions when we go to destroy the sreq, since __del__
        # swallows exceptions, we have to call destroy directly
        sreq.destroy()



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals
import random
import string

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest, skip_if_not_root
from tests.support.unit import skipIf

# Import Salt libs
from salt.ext import six
from salt.ext.six.moves import range
import salt.utils.files
import salt.utils.platform
import salt.utils.stringutils

if not salt.utils.platform.is_windows():
    import grp


@skip_if_not_root
@destructiveTest
class GroupModuleTest(ModuleCase):
    '''
    Validate the linux group system module
    '''

    def setUp(self):
        '''
        Get current settings
        '''
        super(GroupModuleTest, self).setUp()
        self._user = self.__random_string()
        self._user1 = self.__random_string()
        self._no_user = self.__random_string()
        self._group = self.__random_string()
        self._no_group = self.__random_string()
        self.os_grain = self.run_function('grains.item', ['kernel'])
        self._gid = 64989 if 'Windows' not in self.os_grain['kernel'] else None
        self._new_gid = 64998 if 'Windows' not in self.os_grain['kernel'] else None
        if self.os_grain['kernel'] not in ('Linux', 'Windows'):
            self.skipTest(
                'Test not applicable to \'{kernel}\' kernel'.format(
                    **self.os_grain
                )
            )

    def tearDown(self):
        '''
        Reset to original settings
        '''
        self.run_function('user.delete', [self._user])
        self.run_function('user.delete', [self._user1])
        self.run_function('group.delete', [self._group])

    def __random_string(self, size=6):
        '''
        Generates a random names
        '''
        return 'tg-' + ''.join(
            random.choice(string.ascii_lowercase + string.digits)
            for x in range(size)
        )

    def __get_system_group_gid_range(self):
        '''
        Returns (SYS_GID_MIN, SYS_GID_MAX)
        '''
        try:
            login_defs = {}
            with salt.utils.files.fopen('/etc/login.defs') as defs_fd:
                for line in defs_fd:
                    line = salt.utils.stringutils.to_unicode(line).strip()
                    if line.startswith('#'):
                        continue
                    try:
                        key, val = line.split()
                    except ValueError:
                        pass
                    else:
                        login_defs[key] = val
        except OSError:
            login_defs = {'SYS_GID_MIN': 101,
                          'SYS_GID_MAX': 999}

        gid_min = login_defs.get('SYS_GID_MIN', 101)
        gid_max = login_defs.get('SYS_GID_MAX',
                                 int(login_defs.get('GID_MIN', 1000)) - 1)

        return int(gid_min), int(gid_max)

    def __get_free_system_gid(self):
        '''
        Find a free system gid
        '''

        gid_min, gid_max = self.__get_system_group_gid_range()

        busy_gids = [x.gr_gid
                     for x in grp.getgrall()
                     if gid_min <= x.gr_gid <= gid_max]

        # find free system gid
        for gid in range(gid_min, gid_max + 1):
            if gid not in busy_gids:
                return gid

    @destructiveTest
    def test_add(self):
        '''
        Test the add group function
        '''
        # add a new group
        self.assertTrue(self.run_function('group.add', [self._group], gid=self._gid))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['gid'], self._gid)
        self.assertEqual(group_info['name'], self._group)
        # try adding the group again
        if self.os_grain['kernel'] == 'Windows':
            add_group = self.run_function('group.add', [self._group], gid=self._gid)
            self.assertEqual(add_group['result'], None)
            self.assertEqual(add_group['comment'], 'The group {0} already exists.'.format(self._group))
            self.assertEqual(add_group['changes'], [])
        else:
            self.assertFalse(self.run_function('group.add', [self._group], gid=self._gid))

    @destructiveTest
    @skipIf(salt.utils.platform.is_windows(), 'Skip on Windows')
    def test_add_system_group(self):
        '''
        Test the add group function with system=True
        '''

        gid_min, gid_max = self.__get_system_group_gid_range()

        # add a new system group
        self.assertTrue(self.run_function('group.add',
                                          [self._group, None, True]))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['name'], self._group)
        self.assertTrue(gid_min <= group_info['gid'] <= gid_max)
        # try adding the group again
        self.assertFalse(self.run_function('group.add',
                                           [self._group]))

    @destructiveTest
    @skipIf(salt.utils.platform.is_windows(), 'Skip on Windows')
    def test_add_system_group_gid(self):
        '''
        Test the add group function with system=True and a specific gid
        '''

        gid = self.__get_free_system_gid()

        # add a new system group
        self.assertTrue(self.run_function('group.add',
                                          [self._group, gid, True]))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['name'], self._group)
        self.assertEqual(group_info['gid'], gid)
        # try adding the group again
        self.assertFalse(self.run_function('group.add',
                                           [self._group, gid]))

    @destructiveTest
    def test_delete(self):
        '''
        Test the delete group function
        '''
        self.assertTrue(self.run_function('group.add', [self._group]))

        # correct functionality
        self.assertTrue(self.run_function('group.delete', [self._group]))

        # group does not exist
        if self.os_grain['kernel'] == 'Windows':
            del_group = self.run_function('group.delete', [self._no_group])
            self.assertEqual(del_group['changes'], [])
            self.assertEqual(del_group['comment'], 'The group {0} does not exists.'.format(self._no_group))
        else:
            self.assertFalse(self.run_function('group.delete', [self._no_group]))

    def test_info(self):
        '''
        Test the info group function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        group_info = self.run_function('group.info', [self._group])

        self.assertEqual(group_info['name'], self._group)
        self.assertEqual(group_info['gid'], self._gid)
        self.assertIn(self._user, str(group_info['members']))

    @skipIf(salt.utils.platform.is_windows(), 'gid test skipped on windows')
    def test_chgid(self):
        '''
        Test the change gid function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.assertTrue(self.run_function('group.chgid', [self._group, self._new_gid]))
        group_info = self.run_function('group.info', [self._group])
        self.assertEqual(group_info['gid'], self._new_gid)

    def test_adduser(self):
        '''
        Test the add user to group function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.run_function('user.add', [self._user])
        self.assertTrue(self.run_function('group.adduser', [self._group, self._user]))
        group_info = self.run_function('group.info', [self._group])
        self.assertIn(self._user, str(group_info['members']))
        if self.os_grain['kernel'] == 'Windows':
            no_group = self.run_function('group.adduser', [self._no_group, self._no_user])
            no_user = self.run_function('group.adduser', [self._group, self._no_user])
            funcs = [no_group, no_user]
            for func in funcs:
                self.assertIn('Fail', func['comment'])
                self.assertFalse(func['result'])
        else:
            # try add a non existing user
            self.assertFalse(self.run_function('group.adduser', [self._group, self._no_user]))
            # try add a user to non existing group
            self.assertFalse(self.run_function('group.adduser', [self._no_group, self._user]))
            # try add a non existing user to a non existing group
            self.assertFalse(self.run_function('group.adduser', [self._no_group, self._no_user]))

    def test_deluser(self):
        '''
        Test the delete user from group function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        self.assertTrue(self.run_function('group.deluser', [self._group, self._user]))
        group_info = self.run_function('group.info', [self._group])
        self.assertNotIn(self._user, str(group_info['members']))

    def test_members(self):
        '''
        Test the members function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.run_function('user.add', [self._user])
        self.run_function('user.add', [self._user1])
        m = '{0},{1}'.format(self._user, self._user1)
        ret = self.run_function('group.members', [self._group, m])
        if salt.utils.platform.is_windows():
            self.assertTrue(ret['result'])
        else:
            self.assertTrue(ret)
        group_info = self.run_function('group.info', [self._group])
        self.assertIn(self._user, str(group_info['members']))
        self.assertIn(self._user1, str(group_info['members']))

    def test_getent(self):
        '''
        Test the getent function
        '''
        self.run_function('group.add', [self._group], gid=self._gid)
        self.run_function('user.add', [self._user])
        self.run_function('group.adduser', [self._group, self._user])
        ginfo = self.run_function('user.getent')
        self.assertIn(self._group, six.text_type(ginfo))
        self.assertIn(self._user, six.text_type(ginfo))
        self.assertNotIn(self._no_group, six.text_type(ginfo))
        self.assertNotIn(self._no_user, six.text_type(ginfo))
